﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Threading.Tasks;
using System.Reflection;
using System.Configuration;

namespace SqlBulkInsert {
	public class Main {

		#region Declarations

		private List<string> _columnNames = new List<string>();
		private SqlConnection _cn = new SqlConnection();
		public event EventHandler<SqlRowsCopiedEventArgs> RowsInserted;

		#endregion Declarations

		#region Reflection Methods

		public String GetTableNameFromObject<T>() {
			Type attribType = typeof( DataTableAttribute );
			object[] arrTableAttrib = null;
			arrTableAttrib = typeof( T ).GetCustomAttributes( typeof( DataTableAttribute ), true );
			return ( ( DataTableAttribute ) arrTableAttrib[ 0 ] ).TableName;
		}

		public DataColumnAttribute GetColumnNameFromProperty<T>( String propertyName ) {
			DataColumnAttribute propAttr = null;
			object[] arrColumnAttributes = null;

			foreach ( PropertyInfo pi in typeof( T ).GetProperties() ) {
				if ( pi.Name == propertyName ) {
					arrColumnAttributes = pi.GetCustomAttributes( typeof( DataColumnAttribute ), true );
					if ( arrColumnAttributes.Length > 0 ) {
						propAttr = ( DataColumnAttribute ) arrColumnAttributes[ 0 ];
						break;
					}
				}
			}

			return propAttr;
		}

		#endregion Reflection Methods

		#region Methods

		public void Process() {
			// Retrieve the data.
			_cn.ConnectionString = ConfigurationManager.AppSettings[ "DBConnection" ];
			DataTable dt = new DataTable();
			SqlCommand cmd = new SqlCommand("SELECT ID, Desc, UseCode, Active, UpdateDate FROM SomeTable", _cn);
			SqlDataAdapter da = new SqlDataAdapter( cmd );
			da.Fill( dt );
			// Fill the DataObject list.
			try {
				List<DataObject> myList = FillDataObjectList( dt );
				List<String> columnList = new List<string>();
				columnList.Add( "ID" );
				columnList.Add( "Description" );
				columnList.Add( "UseCode" );
				columnList.Add( "Active" );
				columnList.Add( "UpdateDate" );
				// Execute the bulk insert
				BulkInsertDataObjectList<DataObject>( myList, columnList );
			}
			catch ( Exception ex ) {
				Console.Write( String.Format( "Error Occurred:  {0}", ex.Message ) );
			}
		}
		public List<DataObject> FillDataObjectList( DataTable dt ) {
			try {
				List<DataObject> myList = new List<DataObject>();
				foreach ( DataRow row in dt.Rows ) {
					DataObject myObject = new DataObject();
					myObject.ID = Convert.ToInt32( row[ "ID" ] );
					myObject.Description = row[ "Desc" ].ToString();
					myObject.UseCode = Convert.ToInt32( row[ "UseCode" ] );
					myObject.Active = Convert.ToBoolean( row[ "Active" ] );
					myObject.UpdateDate = Convert.ToDateTime( row[ "UpdateDate" ] );
					// The description has leading spaces so we remove them.
					myObject.Description = myObject.Description.TrimStart();
					myList.Add( myObject );
				}
				return myList;
			}
			catch ( Exception ex ) {
				throw ex;
			}
		}

		public Boolean BulkInsertDataObjectList<T>( List<T> myDataObjectList, List<String> propertyList ) {
			System.Data.DataTable dt = new System.Data.DataTable( GetTableNameFromObject<T>() );
			// Create the datatable from the PandaPriceListEntry object.  This allows changes
			// to be made in the data object and flow to this controller.
			PropertyInfo[] props = typeof( T ).GetProperties();
			for ( Int32 i = 0; i == propertyList.Count - 1; i++ ) {
				DataColumnAttribute col = GetColumnNameFromProperty<T>( propertyList[ i ] );
				dt.Columns.Add( col.ColumnName, col.ColumnType );
			}

			// Convert the list to a datatable.
			foreach ( T rec in myDataObjectList ) {
				DataRow row = dt.NewRow();
				for ( int x = 0; x == propertyList.Count - 1; x++ ) {
					row[ x ] = typeof( T ).GetProperty( propertyList[ x ] ).GetValue( rec );
				}
				dt.Rows.Add( row );
			}

			// Bulk Insert the datatable to the database.
			using ( SqlConnection connection =
			new SqlConnection( ConfigurationManager.AppSettings[ "DBConnection" ] ) ) {
				// make sure to enable triggers
				// more on triggers in next post
				SqlBulkCopy bulkCopy =
					new SqlBulkCopy( connection );

				// Set up the bulk copy mappings
				for ( Int32 z = 0; z == propertyList.Count - 1; z++ ) {
					bulkCopy.ColumnMappings.Add( z, z + 1 );
				}

				// Set the destination table name
				bulkCopy.DestinationTableName = GetTableNameFromObject<DataObject>();
				connection.Open();

				// write the data in the dataTable
				bulkCopy.BatchSize = 2000;
				bulkCopy.NotifyAfter = 50000;
				bulkCopy.SqlRowsCopied += new SqlRowsCopiedEventHandler( bulkCopy_SqlRowsCopied );
				bulkCopy.WriteToServer( dt );
				connection.Close();
			}

			return true;
		}

		#endregion Methods

		#region Events

		/// <summary>
		/// Handles the SqlRowsCopied event of the bulkCopy control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Data.SqlClient.SqlRowsCopiedEventArgs"/> instance containing the event data.</param>
		void bulkCopy_SqlRowsCopied( object sender, SqlRowsCopiedEventArgs e ) {
			EventHandler<SqlRowsCopiedEventArgs> temp = RowsInserted;
			if ( temp != null ) {
				temp( this, e );
				Console.Write( String.Format( "{0} - Rows inserted.", e.RowsCopied.ToString() ) );
			}
		}

		#endregion Events
	}
}
